#!/usr/bin/perl -w
#
# Copyright (c) 2007 VMware, Inc.  All rights reserved.

use strict;
use warnings;

use FindBin;
use lib "$FindBin::Bin/../";

use VMware::VIRuntime;
use VMware::VIExt;
use XML::LibXML;
use AppUtil::VMUtil;
use AppUtil::XMLInputUtil;

sub do_browse;
sub do_get;
sub do_put;

$Util::script_version = "1.0";

my %opts = (
   'datacentername' => {
      type => "=s",
      help => "Name of the datacenter",
      required => 0,
   },
   'datastorename' => {
      type => "=s",
      help => "Name of the datastore",
      required => 0,
   },
   'operation' => {
      type => "=s",
      help => "Type of Operation [Get | Put | Browse]",
      required => 1,
      default => "browse",
   },
   'filetype' => {
      type => "=s",
      help => "[datastore | config | tmp]",
      required => 0,
   },
   'remotepath' => {
      type => "=s",
      help => "Remote path of virtual center or ESX server",
      required => 0,
   },
   'remotefilename' => {
      type => "=s",
      help => "Remote path of virtual center or ESX server",
      required => 0,
   },
   'localpath' => {
      type => "=s",
      help => "localpath",
      required => 0,
   },
);

Opts::add_options(%opts);
Opts::parse();
Opts::validate(\&validate);

Util::connect();
my $operation = Opts::get_option('operation');
if($operation eq 'browse') {
   do_browse();
}
elsif($operation eq 'get') {
   do_get();
}
elsif($operation eq 'put') {
   do_put();
}
Util::disconnect();

sub do_get() {
   my $dcname = Opts::get_option('datacentername');
   my $dsname = Opts::get_option('datastorename');
   my $filetype = Opts::get_option('filetype');
   my $localpath = Opts::get_option('localpath');
   my $remotepath;
   my $mode;
   if($filetype eq 'datastore') {
      $mode = "folder";
      $remotepath = Opts::get_option('remotepath');
      print "\nRetrieving the datastore file $remotepath\n"
   }
   elsif($filetype eq 'config') {
      $mode = "host";
      $remotepath = Opts::get_option('remotefilename');
      print "\nRetrieving the configuration file $remotepath\n"
   }
   my $resp = VIExt::http_get_file($mode,$remotepath,$dsname,$dcname,$localpath);
   check_response($resp);
}

sub do_put() {
   my $dcname = Opts::get_option('datacentername');
   my $dsname = Opts::get_option('datastorename');
   my $filetype = Opts::get_option('filetype');
   my $localpath = Opts::get_option('localpath');
   my $remotepath;
   my $mode;
   if($filetype eq 'datastore') {
      $mode = "folder";
      $remotepath = Opts::get_option('remotepath');
      print "\nCopying the file $localpath to $remotepath under ".
            "datastore $dsname abd datacenter $dcname\n";
      my $resp = VIExt::http_put_file($mode,$localpath,$remotepath,$dsname,$dcname);
   }
   elsif($filetype eq 'config') {
      $mode = "host";
      $remotepath = Opts::get_option('remotefilename');
      print "\nCopying the configuration file $remotepath\n";
      my $resp = VIExt::http_put_file($mode,$localpath,$remotepath,$dsname,$dcname);
   }
   elsif($filetype eq 'tmp') {
      $mode = 'tmp';
      print "\nCopying the file $localpath to tmp directory\n";
      $remotepath = Opts::get_option('remotefilename');
      my $resp = VIExt::http_put_tmp_file($localpath,$remotepath);
   }
   print "\nPut operation completed successfully\n";
}

sub do_browse() {
   my $api_type = get_api_type();
   my $url = Opts::get_option('url');
   my $lind = index(Opts::get_option ('url'), "/sdk");
   my $base_url = substr(Opts::get_option ('url'),0,$lind);

   my $filetype = Opts::get_option('filetype');
   my @vm_files;
   if($filetype eq 'datastore') {
      my $dcname;
      if($api_type eq 'HostAgent') {
         $dcname = "ha-datacenter";
      }
      else {
         if(Opts::option_is_set('datacentername')) {
            $dcname = Opts::get_option('datacentername');
         }
         else {
             print "Datacentername is mandatory argument for the browse operation on Virtual Center";
             return;
         }
      }
      my $dsname = Opts::get_option('datastorename');
      my $remotepath = Opts::get_option('remotepath');
      my $temp = "";
      my $folder = "";
      while(1) {
         @vm_files = get_directory_listing(
                     $base_url."/folder/".$remotepath."?dcPath=".$dcname."&dsName=".$dsname);
         if(!defined $vm_files[0]) {
            print "\n###############################################\n";
            print "Location " . $base_url."/folder/".$remotepath."?dcPath=".$dcname."&dsName=".$dsname ."\n\n"; 
            print "No files found under this location\n";
            print "\n###############################################\n";            
         }
         elsif(index($vm_files[0], "##ERROR##") == -1) {
            print "\n###############################################\n";
            print "Location " . $base_url."/folder/".$remotepath."?dcPath=".$dcname."&dsName=".$dsname ."\n\n";
            foreach (@vm_files) {
               print  $_."\n";
            }
            if ($#{@vm_files} == 0) {
               print "No files found under this location\n";
               return;
            }
            print "\n###############################################\n";
            $temp = $remotepath;
         }
         else {
            $remotepath = $temp;
         }
         print "Enter the Name of the folder (enter -1 to exit) ";
         $folder = "";
         $folder = <STDIN>;
         chomp $folder;
         if($folder eq '-1') {
            return;
         }
         $remotepath = $remotepath . "/" . $folder;
      }
   }
   else {
      @vm_files = get_directory_listing($base_url."/host");
      print "\n###############################################\n";
      print "Host Configuration Files\n";
      foreach (@vm_files) {
         print  $_."\n";
      }
      print "\n###############################################\n";
   }
}

sub check_response {
   my ($resp) = @_;
   if ($resp) {
      if (!$resp->is_success) {
         my $ind = index($resp->status_line,'404 Not Found');
         if($ind != -1) {
           print "\nService Not Available\n";
         }
      }
      else {
         print "\nGet operation completed successfully\n";
      }
   }
   else {
      print STDERR "\nGET unsuccessful : failed to get response\n";
   }
}

sub get_directory_listing {
   my ($url) = @_;
   my @listFiles;
   my $service = Vim::get_vim_service();
   my $userAgent = $service->{vim_soap}->{user_agent};
   my $request = HTTP::Request->new("GET", $url);
   my $response = $userAgent->request($request);
   if ($response->is_success) {
      my $content = $response->content;
      my $count = 0;
      while($content =~ m/(<a href=\".*\">.*<\/a>)/g) {
         my $sind = index($1,'>')+1;
         my $name = substr($1,$sind);
         my $lind = index($name,"</a>");
         $name = substr($name,0,$lind);
         $listFiles[$count] = $name;
         $count = $count + 1;
      }
   } else {
      print "Error : " . $response->status_line . "\n";
      $listFiles[0] = "##ERROR## ". $response->status_line;
   }
   return @listFiles;
}

sub get_api_type() {
   my $si_moref = ManagedObjectReference->new(type => 'ServiceInstance',
                                              value => 'ServiceInstance');
   my $si_view = Vim::get_view(mo_ref => $si_moref);
   my $api_type = $si_view->content->about->apiType;
   return $api_type;
}

sub validate {
   my $valid = 1;
   my $operation = Opts::get_option('operation');
   my $filetype = Opts::get_option('filetype');
   my $remotepath = Opts::get_option('remotepath');
   my $remotefilename = Opts::get_option('remotefilename');
   my $localpath = Opts::get_option('localpath');
   my $datastore = Opts::get_option('datastorename');
   my $datacenter = Opts::get_option('datacentername');
   
   if($operation eq 'get') {
      if(defined $filetype) {
         if($filetype eq 'datastore') {
            if(!(defined $remotepath && defined $localpath
                 && defined $datastore && defined $datacenter)) {
               $valid = 0;
               print "\nFor get operation on datastore files remotepath,".
                     " localpath, datacentername and datastorename are ".
                     "the manadatory arguments.\n"
            }
         }
         elsif($filetype eq 'config') {
            if(!(defined $remotefilename && defined $localpath)) {
               $valid = 0;
               print "\nFor get operation on config files remotefilename and".
                     " localpath are the manadatory arguments.\n"
            }
         }
         elsif($filetype eq 'tmp') {
            $valid = 0;
            print "\nOnly put operation is supported on tmp directory\n";
         }
         else {
            $valid = 0;
            print "\nFor the get operation only valid vaues for filetype ".
                  "argument are datastore and config\n";
         }
      }
      else {
         $valid = 0;
         print "\nfiletype is the mandatory option for the get operation\n";
      }
   }
   elsif($operation eq 'put') {
      if(defined $filetype) {
         if($filetype eq 'datastore') {
            if(!(defined $remotepath && defined $localpath
                 && defined $datastore && defined $datacenter)) {
               $valid = 0;
               print "\nFor put operation on datastore files remotepath,".
                     " localpath, datastorename and datacentername are the ".
                     "manadatory arguments.\n"
            }
         }
         elsif($filetype eq 'config') {
            if(!(defined $remotefilename && defined $localpath)) {
               $valid = 0;
               print "\nFor put operation on config files remotefilename and".
                     " localpath are the manadatory arguments.\n"
            }
         }
         elsif($filetype eq 'tmp') {
            if(!(defined $remotefilename && defined $localpath)) {
               $valid = 0;
               print "\nFor put operation on tmp directory remotefilename and".
                     " localpath are the manadatory arguments.\n"
            }
         }
         else {
            $valid = 0;
            print "\nFor the put operation only valid vaues for filetype ".
                  "argument are datastore, config and tmp\n";
         }
      }
      else {
         $valid = 0;
         print "\nfiletype is the mandatory option for the put operation\n";
      }
   }
   elsif($operation eq 'browse') {
      if(defined $filetype) {
         if($filetype eq 'datastore') {
            if(!(defined $remotepath && defined $datastore)) {
               $valid = 0;
               print "\nFor browse operation on datastore folders remotepath,".
                     " datastorename are ".
                     "the manadatory arguments.\n"
            }
         }         
         elsif($filetype eq 'tmp') {
            $valid = 0;
            print "\nOnly browse operation is supported on tmp directory\n";
         }
         elsif($filetype ne 'config') {
            $valid = 0;
            print "\nFor the browse operation only valid values for filetype ".
                  "argument are datastore and config\n";
         }
      }
      else {
         $valid = 0;
         print "\nfiletype is the mandatory option for the browse operation\n";
      }
   }
   else {
      $valid = 0;
      print "\nInvalid operation: $operation\n";
      print "List of valid operations:\n";
      print "get\n";
      print "put\n";
      print "browse\n";
   }
   return $valid;
}

=head1 NAME

fileaccess.pl - Allow the get and put operations on datastore and configuration files and put 
operation on the /tmp directory.

=head1 SYNOPSIS

fileaccess.pl --operation<put|get|browse>[options]

=head1 DESCRIPTION

This VI Perl command-line utility provides an interface for get and put operations on the datastore 
and configuration files and put operation on the /tmp directory.
The Basic operations are:

 Retrieve the datastore and configuration files.
 Put the files under datastore directory.
 Put the modified configuration files.
 Put the files under /tmp directory.
 Display all the accessible datastore and configuration files.

=head1 OPTIONS

=head2 GENERAL OPTIONS

=over

=item B<operation>

 Operation to be performed.  Must be one of the following:
  I<get> (Retrieving the datastore and configuration files.),
  I<put> (Putting the files under datastore and tmp directory, modifying the configuration files),
  I<browse> (Displaying all the accessible datastore and configuration files.),

=item B<datacentername>

Optional. Name of the Datacenter. Mandatory argument for get and put operations on datastore files.

=item B<datastorename>

Optional. Name of the Datastore. Mandatory argument for get and put operations on datastore files.

=item B<filetype>

Optional. Must be [datastore | config | tmp]

   Value must be datastore for the get and put operations on datastore files.
   Value must be config for the get and put operations on configuration files.
   Value must be tmp for put operation on /tmp directory.

=item B<remotepath>

Optional. Remote path of the datastore file.
Mandatory argument for the get and put operations on datastore files

=item B<remotefilename>

Optional. Configuration file name.
Mandatory argument for the get and put operations on configuration files

=item B<localpath>

Optional. Localpath to save the extracted log.

=back

=head1 EXAMPLES

To retrieve the datastore file

   fileaccess.pl --url <https://<IP Address>:<Port>/sdk/vimService>
                 --username myuser --password mypassword --operation get
                 --localpath "C:/tmp/vm.vmx" --remotepath "vm/vm.vmx"
                 --filetype datastore --datastorename <datastore name>
                 -- datacentername <datacenter name>

To retrieve the configuration file

   fileaccess.pl --url <https://<IP Address>:<Port>/sdk/vimService>
                 --username myuser --password mypassword --operation get
                 --localpath "C:/tmp/vm.vmx"
                 --remotefilename <name of configuration files> 
                 --filetype config

To put the datastore file

   fileaccess.pl --url <https://<IP Address>:<Port>/sdk/vimService>
                 --username myuser --password mypassword --operation put
                 --localpath "C:/tmp/vm.vmx" --remotepath "vm/vm.vmx"
                 --filetype datastore  --datastorename <datastore name>
                 -- datacentername <datacenter name>

To put the configuration file

   fileaccess.pl --url <https://<IP Address>:<Port>/sdk/vimService>
                 --username myuser --password mypassword --operation put
                 --localpath "C:/tmp/filename" --remotefilename <name of configuration files> 
                 --filetype config

To put the file under tmp directory

   fileaccess.pl --url <https://<IP Address>:<Port>/sdk/vimService>
                 --username myuser --password mypassword --operation put
                 --localpath "C:/tmp/filename" --remotefilename <name of configuration files> 
                 --filetype tmp

To display the accessible datastore files.

   fileaccess.pl --url <https://<IP Address>:<Port>/sdk/vimService>
                 --username myuser --password mypassword --operation browse --filetype datastore
                 --remotepath vm --datastorename <Datastore name> --datacentername <Datacenter name>

To display the accessible configuration files.

   fileaccess.pl --url <https://<IP Address>:<Port>/sdk/vimService>
                 --username myuser --password mypassword --operation browse --filetype config

=head1 SUPPORTED PLATFORMS

All operations work with VMware VirtualCenter 2.5.

All operations work with VMware ESX Server 3.5.